package fuzzyMaghribis;

import org.nzdis.micro.MicroMessage;

public class MessageFactory {

	public static final String CHEATER_KEY = "CHEATER";
	public static final String EMPLOYMENT_PAYMENT = "PAYMENT";
	public static final String GOODS_TRANSFER = "GOODS_TRANSFER";
	public static final String GOODS_VALUE = "GOODS_VALUE";
	public static final String TRADE_CONFIRMATION = "TRADE_CONFIRMATION";
	public static final String TRADE_RESULT = "TRADE_RESULT";
	public static final String CHEATER_ANNOUNCEMENT = "CHEATER_ANNOUNCEMENT";
	public static final String NON_CHEATER_ANNOUNCEMENT = "NON_CHEATER_ANNOUNCEMENT";
	//validation indicator if agent really has cheated - simplifies stats but not used in code
	public static final String CHEATING_INDICATOR = "CHEATING_INDICATOR";
	//how much was cheating?
	public static final String CHEATING_EXTENT = "CHEATING_EXTENT";
	
	public static MicroMessage makeCheaterInfo(String cheaterName){
		MicroMessage message = new MicroMessage();
		message.setPerformative(CHEATER_ANNOUNCEMENT);
		message.setConversationID("CHEAT_ID");
		message.put(CHEATER_KEY, String.valueOf(cheaterName));
		return message;
	}
	
	public static MicroMessage makeNonCheaterInfo(String nonCheaterName){
		MicroMessage message = new MicroMessage();
		message.setPerformative(NON_CHEATER_ANNOUNCEMENT);
		message.setConversationID("CHEAT_ID");
		message.put(CHEATER_KEY, String.valueOf(nonCheaterName));
		return message;
	}
	
	/**
	 * Creates trade confirmation message.
	 * @param employer Employer who will receive the message
	 * @param value return achieved
	 * @param convId conversation id
	 * @param cheated indicator if seller has cheated
	 * @param cheatingExtent if so, by how much - positive value
	 * @return
	 */
	public static MicroMessage makeTradeConfirmation(String employer, Float value, String convId, Boolean cheated, Float cheatingExtent){
		MicroMessage message = new MicroMessage();
		message.setPerformative(TRADE_CONFIRMATION);
		message.set(TRADE_RESULT, value);
		message.setRecipient(String.valueOf(employer));
		message.setConversationID(convId);
		message.set(CHEATING_INDICATOR, cheated.toString());
		message.set(CHEATING_EXTENT, cheatingExtent.toString());
		return message;
	}
	
	public static MicroMessage makeSendGoodsOneOffMessage(String employee, Float goods){
		MicroMessage message = new MicroMessage();
		message.setPerformative(GOODS_TRANSFER);
		message.setRecipient(String.valueOf(employee));
		message.set(GOODS_VALUE, goods);
		return message;
	}
	
	public static boolean isCheaterInfo(MicroMessage message){
		return message.getPerformative().equals(CHEATER_ANNOUNCEMENT);
	}
	
	public static boolean isNonCheaterInfo(MicroMessage message){
		return message.getPerformative().equals(NON_CHEATER_ANNOUNCEMENT);
	}
	
	public static boolean isTradeConfirmation(MicroMessage message){
		return message.getPerformative().equals(TRADE_CONFIRMATION);
	}
	
	public static boolean isGoodsTransferMessage(MicroMessage message){
		return message.getPerformative().equals(GOODS_TRANSFER);
	}
	
}
