package fuzzyMaghribis;

import org.sofosim.environment.stats.Statistics;
import org.sofosim.environment.stats.StringStatsCalculator;
import org.sofosim.environment.stats.charts.DistributionChart;
import org.sofosim.environment.stats.printer.GraphsPrinter;
import org.sofosim.environment.stats.printer.StatsWriter;
import fuzzyMaghribis.network.GraphInformation;
import fuzzyMaghribis.network.Network;

public class Stats extends Statistics{

	public static final String CHART_CHEATER_ADVISOR_PERCEPTION = "Cheater and Advisor Perception";
	public static final String CHART_ONE_OFF_EMPLOYMENT = "Employment in OneOff trading";
	public static final String CHART_QUOTAS_ONE_OFF = "Quotas in OneOff relationships";
	
	public static final String KEY_EMPLOYED_AGENTS = "Employed Sellers";
	public static final String KEY_EMPLOYING_TRADERS = "Employing Traders";
	public static final String KEY_AVG_WAGE_EMPLOYEES = "Average of Employees' Wages";
	public static final String KEY_AVG_WAGE_OFFERED = "Average Wage Offered";

	//number of agents that cheated in this round
	public static final String KEY_CHEATED = "Cheated this round";
	//number of traders that have been cheated this round
	public static final String KEY_HAVE_BEEN_CHEATED = "Have been cheated";

	public static final String KEY_NUMBER_OF_AGENTS_WITHOUT_PAST_EMPLOYMENT = "Number of agents that have never been employed";
	
	public static final String KEY_AVG_CHEATER_MEMORY = "Mean Cheater memory";
	public static final String KEY_DEV_CHEATER_MEMORY = "Std. Dev. Cheater memory";
	public static final String KEY_AVG_ADVISOR_MEMORY = "Mean Advisor memory";
	public static final String KEY_DEV_ADVISOR_MEMORY = "Std. Dev. Advisor memory";
	
	public static final String KEY_ONE_OFF_EMPLOYED = "One Off employed";
	public static final String KEY_CHEATING_QUOTA_OF_ONE_OFF_EMPLOYED = "Cheating quota of one-off employed sellers";
	public static final String KEY_ADVICE_TEST_PROB = "Advice testing probability";
	public static final String KEY_ADVISE_TRUTHFULLY_PROB = "Advise truthfully probability";
	public static final String KEY_POTENTIAL_CHEATERS_IN_ONE_OFF_EMPLOYMENTS = "Cheaters in employment (not necessarily cheating)";
	public static final String KEY_ADVISED_TRUTHFULLY = "Advised truthfully";
	public static final String KEY_ADVISED_NON_TRUTHFULLY = "Advised non-truthfully";
	public static final String KEY_GOT_GOOD_ADVICE = "Got good advice";
	public static final String KEY_GOT_BAD_ADVICE = "Got bad advice";
	public static final String KEY_AVG_CHEATING_PROB_OF_EMPLOYED_POTENTIAL_CHEATERS = "Cheating probability of employed potential cheaters";
	public static final String KEY_NUMBER_OF_ADVICE_REQUESTS_THIS_ROUND = "Number of advice requests";
	
	public static final String KEY_TRADER_INFO_NETWORK = "Trader Information Network";
	
	DistributionChart cheaterTrustHistogram = null;
	DistributionChart advisorTrustHistogram = null;
	
	DistributionChart traderNetworkDegreeDistributionHistogram = null;
	
	public boolean debugTiming = false;
	
	Stats(boolean activateGraphController, boolean useMapInListener,
			int schedulingInterval) {
		super(activateGraphController, useMapInListener, schedulingInterval);
		GraphsPrinter.enabled = true;
		StatsWriter.enabled = true;	
		COLLECT_DATA_EVERY_X_ROUNDS = 50;
		RUN_CUSTOM_METHOD_EVERY_X_ROUNDS = 50;
		RUN_CUSTOM_METHOD_LAST = true;	
	}

	
	private static int cheated = 0;
	private static int firedCheater = 0;
	private static int correctlyClassifiedCheaters = 0;
	private static int falselyClassifiedNonCheaters = 0;
	private static int haveOneOffEmployment = 0;
	private static int haveOneOffEmploymentAndCheater = 0;
	private static int advisedTruthfully = 0;
	private static int advisedNotTruthfully = 0;
	private static int gotGoodAdvice = 0;
	private static int gotBadAdvice = 0;
	
	private static int numberOfAdviceRequestsThisRound = 0;
	
	private static StringStatsCalculator sanctionsStats;
	
	public static void cheated(){
		cheated++;
	}
	
	public static void detectedCheaterCorrectly(){
		correctlyClassifiedCheaters++;
	}
	
	public static void falseDetectionOfCheater(){
		falselyClassifiedNonCheaters++;
	}

	public static void haveOneOffEmployment(){
		haveOneOffEmployment++;
	}
	
	public static void haveOneOffEmploymentAndAmCheaterByNature(Agent ag){
		haveOneOffEmploymentAndCheater++;
		statsCalculator.enterValue(KEY_AVG_CHEATING_PROB_OF_EMPLOYED_POTENTIAL_CHEATERS, ag.cheatingProb.doubleValue());
	}
	
	public static void gotGoodAdvice(){
		gotGoodAdvice++;
	}
	
	public static void gotBadAdvice(){
		gotBadAdvice++;
	}
	
	public static void numberOfAdviceRequestsSent(int numberOfRequests){
		numberOfAdviceRequestsThisRound += numberOfRequests;
	}
	
	
	//switch to check if live-graph-specific header has been written
	boolean headerWritten = true;
	
	@Override
	public void singleStep() {
		if(debugTiming){
			System.out.println("Stats called.");
		}
		if(!headerWritten){
			writeDirectlyToFile("##|##\n");
			headerWritten = true;
		}

		if(sim.graphHandler != null){
			//if updating Forces graph by Mason scheduler, set override to true
			if(debugTiming){
				System.out.print("Doing vis....");
			}
			sim.graphHandler.visualizeGraphs(sim.usingMasonSchedulerForGraphs());
			if(debugTiming){
				System.out.println(" done.");
			}
		}
		
		
		int numberOfEmployedAgents = 0;
		int numberOfEmployingTraders = 0;
		int numberOfPartiallyEmployingTraders = 0;
		int numberOfAgentsWithoutPastEmployment = 0;
		int employeeWageSum = 0;
		int employerOfferedWageSum = 0;
		int partiallyEmployingEmployerOfferedWageSum = 0;
		double employmentProbabilitySum = 0;
		int numberOfPotentialCheaters = 0;
		int numberOfUnemployedCheaters = 0;
		int numberOfEmployedCheaters = 0;
		
		if(debugTiming){
			System.out.print("Clearing...");
		}
		
		statsCalculator.clearEntries(KEY_AVG_CHEATER_MEMORY);
		statsCalculator.clearEntries(KEY_DEV_CHEATER_MEMORY);
		statsCalculator.clearEntries(KEY_AVG_ADVISOR_MEMORY);
		statsCalculator.clearEntries(KEY_DEV_ADVISOR_MEMORY);
		statsCalculator.clearEntries(KEY_ADVISE_TRUTHFULLY_PROB);
		statsCalculator.clearEntries(KEY_ADVICE_TEST_PROB);
		
		if(debugTiming){
			System.out.println("done.");
		}
		
		for(Agent ag: MaghribiSim.agentMap.values()){
			if(!ag.employerWageList.isEmpty()){
				//employed
				numberOfEmployedAgents++;
				if(ag.cheater){
					numberOfEmployedCheaters++;
				}
			} else {
				//unemployed
				
			}
			if(ag.cheater != null && ag.cheater){
				numberOfPotentialCheaters++;
			}
			//both
			//agents that never had any employment
			if(ag.pastEmployers.isEmpty() && ag.employerWageList.isEmpty()){
				numberOfAgentsWithoutPastEmployment++;
			}
		}
		
		if(debugTiming){
			System.out.println("Start of cheater and advisor stats.");
		}
		for(Trader ag: MaghribiSim.traderMap.values()){
			//Traders
			statsCalculator.enterValue(KEY_AVG_CHEATER_MEMORY, ag.cheaterMemory.getMeanOfAllEntries());
			statsCalculator.enterValue(KEY_DEV_CHEATER_MEMORY, ag.cheaterMemory.getStandardDeviationOfAllEntries().doubleValue());
			statsCalculator.enterValue(KEY_AVG_ADVISOR_MEMORY, ag.advisorMemory.getMeanOfAllEntries());
			statsCalculator.enterValue(KEY_DEV_ADVISOR_MEMORY, ag.advisorMemory.getStandardDeviationOfAllEntries().doubleValue());
			statsCalculator.enterValue(KEY_ADVISE_TRUTHFULLY_PROB,  ag.adviseTruthfullyProbability.doubleValue());
			statsCalculator.enterValue(KEY_ADVICE_TEST_PROB,  ag.adviceTestProbability.doubleValue());
			
			//both
			if(ag.employmentProb != null){
				employmentProbabilitySum += ag.employmentProb;
			}
		}
		if(debugTiming){
			System.out.println("End of cheater and advisor stats.");
		}
		if(numberOfEmployedAgents != 0 || MaghribiSim.oneOffTrades){
			
			boolean print = true;
			
			int numberOfNonEmployingTraders = MaghribiSim.NO_OF_TRADERS - numberOfEmployingTraders;

			addDataSeriesEntryForCurrentRound(CHART_ONE_OFF_EMPLOYMENT, KEY_HAVE_BEEN_CHEATED, correctlyClassifiedCheaters, print);
			
			addDataSeriesEntryForCurrentRound(CHART_CHEATER_ADVISOR_PERCEPTION, KEY_AVG_ADVISOR_MEMORY, statsCalculator.getMean(KEY_AVG_ADVISOR_MEMORY), print);
			addDataSeriesEntryForCurrentRound(CHART_CHEATER_ADVISOR_PERCEPTION, KEY_DEV_ADVISOR_MEMORY, statsCalculator.getMean(KEY_DEV_ADVISOR_MEMORY), print);
			addDataSeriesEntryForCurrentRound(CHART_CHEATER_ADVISOR_PERCEPTION, KEY_AVG_CHEATER_MEMORY, statsCalculator.getMean(KEY_AVG_CHEATER_MEMORY), print);
			addDataSeriesEntryForCurrentRound(CHART_CHEATER_ADVISOR_PERCEPTION, KEY_DEV_CHEATER_MEMORY, statsCalculator.getMean(KEY_DEV_CHEATER_MEMORY), print);
			
			addDataSeriesEntryForCurrentRound(CHART_ONE_OFF_EMPLOYMENT, KEY_ONE_OFF_EMPLOYED, haveOneOffEmployment, print);
			addDataSeriesEntryForCurrentRound(CHART_ONE_OFF_EMPLOYMENT, KEY_POTENTIAL_CHEATERS_IN_ONE_OFF_EMPLOYMENTS, haveOneOffEmploymentAndCheater, print);
			addDataSeriesEntryForCurrentRound(CHART_QUOTAS_ONE_OFF, KEY_AVG_CHEATING_PROB_OF_EMPLOYED_POTENTIAL_CHEATERS, statsCalculator.getMean(KEY_AVG_CHEATING_PROB_OF_EMPLOYED_POTENTIAL_CHEATERS), print);
			addDataSeriesEntryForCurrentRound(CHART_QUOTAS_ONE_OFF, KEY_CHEATING_QUOTA_OF_ONE_OFF_EMPLOYED, cheated / (float) haveOneOffEmployment, print, true);
			addDataSeriesEntryForCurrentRound(CHART_ONE_OFF_EMPLOYMENT, KEY_ADVISED_TRUTHFULLY, advisedTruthfully, print);
			addDataSeriesEntryForCurrentRound(CHART_ONE_OFF_EMPLOYMENT, KEY_ADVISED_NON_TRUTHFULLY, advisedNotTruthfully, print);
			addDataSeriesEntryForCurrentRound(CHART_ONE_OFF_EMPLOYMENT, KEY_GOT_GOOD_ADVICE, gotGoodAdvice, print);
			addDataSeriesEntryForCurrentRound(CHART_ONE_OFF_EMPLOYMENT, KEY_GOT_BAD_ADVICE, gotBadAdvice, print);
			
			if(MaghribiSim.adjustAdviceProbabilities){
				addDataSeriesEntryForCurrentRound(CHART_QUOTAS_ONE_OFF, KEY_ADVISE_TRUTHFULLY_PROB, statsCalculator.getMean(KEY_ADVISE_TRUTHFULLY_PROB), print);
				addDataSeriesEntryForCurrentRound(CHART_QUOTAS_ONE_OFF, KEY_ADVICE_TEST_PROB, statsCalculator.getMean(KEY_ADVICE_TEST_PROB), print);
			}
			
			appendToFile(KEY_NUMBER_OF_ADVICE_REQUESTS_THIS_ROUND, String.valueOf(numberOfAdviceRequestsThisRound));

			addToRoundBuffer(new StringBuffer(KEY_EMPLOYED_AGENTS).append(": ").append(numberOfEmployedAgents), true);
			//add to key value statsMap
			addKeyValueToRoundBuffer(new StringBuffer(KEY_EMPLOYING_TRADERS), numberOfEmployingTraders, true, true);
			addToRoundBuffer(new StringBuffer(KEY_NUMBER_OF_AGENTS_WITHOUT_PAST_EMPLOYMENT).append(": ").append(numberOfAgentsWithoutPastEmployment), true);
			addToRoundBuffer(new StringBuffer(KEY_AVG_WAGE_EMPLOYEES).append(": ").append(employeeWageSum / new Double(numberOfEmployedAgents)), true);
			try{
				addToRoundBuffer(new StringBuffer(KEY_AVG_WAGE_OFFERED).append(": for non-employing traders: ").append(employerOfferedWageSum / (float)numberOfNonEmployingTraders)
						.append(" (non-employing traders: ").append(numberOfNonEmployingTraders).append(");\nfor partially employing traders: ")
						.append(partiallyEmployingEmployerOfferedWageSum / (float) numberOfPartiallyEmployingTraders)
						.append(" (partially employing traders: ").append(numberOfPartiallyEmployingTraders).append(")"), true);
			} catch(ArithmeticException e){
				
			}
			
			addToRoundBuffer(new StringBuffer(KEY_CHEATED).append(": ").append(cheated), true);
			
			//show information network measures if available
			addKeyValueToRoundBuffer(MaghribiSim.traderInfoNetwork != null ? MaghribiSim.traderInfoNetwork.getGraphType() : "Uninitialized network", MaghribiSim.traderInfoNetwork != null ? MaghribiSim.traderInfoNetwork.getDegreeInformation() : "", true, false);
			
		
			if(MaghribiSim.oneOffTrades && sim.schedule.getSteps() % 10 == 0){
				//print trust distributions
				if(sim.SHOW_STATS_GRAPHS){
					if(cheaterTrustHistogram == null){
						cheaterTrustHistogram = new DistributionChart(KEY_AVG_CHEATER_MEMORY);
					}
					cheaterTrustHistogram.createDataset(statsCalculator, KEY_AVG_CHEATER_MEMORY, 10);
					cheaterTrustHistogram.prepareGui();
					if(!cheaterTrustHistogram.isVisible()){
						cheaterTrustHistogram.setVisible(true);
						registerForPrinting(cheaterTrustHistogram);
					}
				
					if(statsCalculator.hasKey(KEY_AVG_ADVISOR_MEMORY)){
						if(advisorTrustHistogram == null){
							advisorTrustHistogram = new DistributionChart(KEY_AVG_ADVISOR_MEMORY);
						}
						advisorTrustHistogram.createDataset(statsCalculator, KEY_AVG_ADVISOR_MEMORY, 10);
						advisorTrustHistogram.prepareGui();
						if(!advisorTrustHistogram.isVisible()){
							advisorTrustHistogram.setVisible(true);
							registerForPrinting(advisorTrustHistogram);
						}
					}
					
					Network network = MaghribiSim.traderInfoNetwork;
					if(network != null){
						GraphInformation networkInfo = network.getDegreeInformation();
						if(networkInfo != null){
							if(traderNetworkDegreeDistributionHistogram == null){
								traderNetworkDegreeDistributionHistogram = new DistributionChart("Trader Network Degree Distribution");
							}
							if(networkInfo.degreeDistribution != null || networkInfo.degreeDistributionArray != null){
								if(networkInfo.degreeDistribution != null){
									traderNetworkDegreeDistributionHistogram.createDataset(networkInfo.degreeDistribution, KEY_TRADER_INFO_NETWORK);
								} else {
									traderNetworkDegreeDistributionHistogram.createDataset(networkInfo.degreeDistributionArray, KEY_TRADER_INFO_NETWORK, networkInfo.maxDegree, networkInfo.minDegree, 0, false);
								}
								traderNetworkDegreeDistributionHistogram.prepareGui();
								if(!traderNetworkDegreeDistributionHistogram.isVisible()){
									traderNetworkDegreeDistributionHistogram.setVisible(true);
									registerForPrinting(traderNetworkDegreeDistributionHistogram);
								}
							} else {
								System.out.println("Network distribution info is null.");
							}
						}
					}
				}
			}
			
		}
		cheated = 0;
		correctlyClassifiedCheaters = 0;
		falselyClassifiedNonCheaters = 0;
		haveOneOffEmployment = 0;
		haveOneOffEmploymentAndCheater = 0;
		advisedTruthfully = 0;
		advisedNotTruthfully = 0;
		gotGoodAdvice = 0;
		gotBadAdvice = 0;
		numberOfAdviceRequestsThisRound = 0;
		
		//these are collected over the steps of the agents, thus need to be cleared here, not at the beginning of the stats step (which would clear all inputs from that round)
		statsCalculator.clearEntries(KEY_AVG_CHEATING_PROB_OF_EMPLOYED_POTENTIAL_CHEATERS);
		
		if(debugTiming){
			System.out.println("Stats done.");
		}
	}


	
	@Override
	public void setupCharts() {
		setupChart(CHART_CHEATER_ADVISOR_PERCEPTION, CHART_CHEATER_ADVISOR_PERCEPTION, "Rounds", "Cheater / Advisor Perception");
		setupChart(CHART_ONE_OFF_EMPLOYMENT, CHART_ONE_OFF_EMPLOYMENT, "Rounds", CHART_ONE_OFF_EMPLOYMENT);
		setupChart(CHART_QUOTAS_ONE_OFF, CHART_QUOTAS_ONE_OFF, "Rounds", CHART_QUOTAS_ONE_OFF);
		
	}

	@Override
	public void customMethodThatIsRunPeriodically() {
		
	}

	@Override
	public void printGraphs() {
		
	}

	@Override
	public void reset() {
		
	}

}
