package fuzzyMaghribis.batchSets;

import org.apache.commons.math3.distribution.NormalDistribution;
import org.apache.commons.math3.random.MersenneTwister;
import org.frogberry.windowPositionSaver.PositionSaver;
import org.nzdis.micro.bootloader.MicroBootProperties;
import org.nzdis.micro.constants.MessagePassingFrameworks;
import org.nzdis.micro.inspector.PlatformInspectorGui;
import org.nzdis.micro.inspector.listeners.ShowSelectedAgentsOutputInInspectorGuiListener;
import org.sofosim.batching.BatchedExperiments;
import fuzzyMaghribis.MaghribiSim;
import fuzzyMaghribis.MaghribiSimUi;
import fuzzyMaghribis.NetworkType;
import fuzzyMaghribis.Stats;

public class ScaleFreeBatchRun extends BatchedExperiments{

	public static final boolean showUi = true;
	public static long seed = 4336690L;
	public static int experimentalRounds = 20001;
	
	public static void setupPlatform(){
		MicroBootProperties.setInternalMessageTransport(MessagePassingFrameworks.MICRO_FIBER);
		MicroBootProperties.startLazy(false);
		MicroBootProperties.setSynchronousMessageDeliveryMode(true);
		MicroBootProperties.setAgentConsoleOutputLevel(0);
		MicroBootProperties.setMessageTypePrefixForAgentConsoleOutput(true);
		MicroBootProperties.setMicroMessageValidator("fuzzyMaghribis.ConversationMessageValidator");
		PositionSaver.setWindowSetId("GenoeseNetwork");
		PositionSaver.setIncludeTitleInFrameKeys(false);
		PlatformInspectorGui.getInstance().registerListener(new ShowSelectedAgentsOutputInInspectorGuiListener());
	}
	
	@SuppressWarnings("static-access")
	public static void setupConfiguration(MaghribiSim sim){

		float scaleFactor = 1f;		
		//Merchants per Investor
		Float investorMerchantRatio = 1.57f;
		//general setup
		sim.NO_OF_TRADERS = new Double(180 * scaleFactor).intValue();
		sim.NO_OF_AGENTS = new Float(((float)sim.NO_OF_TRADERS) * investorMerchantRatio).intValue();
		sim.QUOTA_OF_CHEATERS = 0.4f;
		
		//Maghribi version with proactive communication
		sim.cheatingCommunication = false; //proactive
		//makes traders consider own information
		sim.takeAdviceFromTraders = true; //reactive
		//advice-related
		sim.maxAdviceRequestsForOneOffTrades = 10;
		sim.adviceTestProbability = 0.5f;
		sim.adviseTruthfullyProbability = 0.6f;
		sim.ndist = new NormalDistribution(new MersenneTwister(seed), sim.adviseTruthfullyProbability, 0.1f, NormalDistribution.DEFAULT_INVERSE_ABSOLUTE_ACCURACY);
		sim.adjustAdviceProbabilities = true;
		sim.increasedTrustProbabilityForTraderInTraderConnections = false;
		sim.trustIncreaseForKnownTraders = 0.1f;
		sim.correctAdviceToPositivelyRememberedAdvisors = false;
		sim.wattsStrogatzRewiringProbability = 0.3f;
		
		//memory-related
		sim.useDiscreteMemoryInsteadOfDiscountMemory = true;
		//discounting
		sim.CHEATER_MEMORY_DISCOUNT_FACTOR = 0.99f;
		sim.MEMORY_ENTRY_DELETION_THRESHOLD_ABS_VAL_FROM_ZERO = 0.05f;
		//discrete
		sim.cheaterMemoryLengthForDiscreteMemory = 40;
		
		sim.maxNumberOfTraderTrustRelations = 40;
		
		sim.MINIMAL_CHEATING_PROB = 1.0f;
		sim.MAXIMAL_CHEATING_PROB = 1.0f;
		
		int frequency = 500;
		sim.getStatistics().setChartsAutoPrintFrequency(frequency);
		sim.getStatistics().setDataCollectionFrequency(1);
		sim.getStatistics().setChartUpdateFrequency(frequency);
		sim.getStatistics().setStatsFormSavingFrequency(frequency);
		sim.getStatistics().setCustomMethodExecutionFrequency(-1);
		//write out every xx lines (i.e. rounds)
		sim.getStatistics().setNoOfLinesBufferedBeforeWritingToFile(100);
		sim.getStatistics().overwriteOutputFileUponHeaderChange = true;
		sim.getStatistics().showStatsForm = showUi;
		sim.SHOW_STATS_GRAPHS = showUi;
		
		Stats.setGlobalSubfolderForOutput("results/"
				+ MaghribiSim.NO_OF_TRADERS + "_" 
				+ MaghribiSim.NO_OF_AGENTS + "_"
				+ "mg_" + (sim.cheatingCommunication ? "t" : "f") + "_"
				+ "oOff_" + (sim.takeAdviceFromTraders ? "cm" : "noCm") + "_" 
				+ "cht" + sim.QUOTA_OF_CHEATERS + "_fgt" + sim.CHEATER_MEMORY_DISCOUNT_FACTOR 
				+ "_rq" + sim.maxAdviceRequestsForOneOffTrades + "_tstAdv" + sim.adviceTestProbability 
				+ "_tth" + sim.adviseTruthfullyProbability + "_dy" + String.valueOf(sim.adjustAdviceProbabilities).substring(0, 1)
				+ "_incTr" + (sim.increasedTrustProbabilityForTraderInTraderConnections ? sim.trustIncreaseForKnownTraders : "f")
				+ "_reci" + String.valueOf(sim.correctAdviceToPositivelyRememberedAdvisors).substring(0, 1)
				+ "_sd" + String.valueOf(seed).substring(String.valueOf(seed).length() - 2)
				+ (sim.useDiscreteMemoryInsteadOfDiscountMemory ? ("_mem" + sim.cheaterMemoryLengthForDiscreteMemory) : 
					("_del" + sim.MEMORY_ENTRY_DELETION_THRESHOLD_ABS_VAL_FROM_ZERO))
				+ "_wts" + sim.wattsStrogatzRewiringProbability
				+ "_" + sim.maxNumberOfTraderTrustRelations + "nt" + sim.traderInfoNetworkType
				, true);
	}
	

	
	@Override
	public void batchSpecification() {
		MaghribiSimUi ui = null;
		MaghribiSim sim = null;
		
		//RANDOM
		sim = new MaghribiSim(seed, showUi);
		MicroBootProperties.setRandomNumberGeneratorSeed(sim.seed());
		
		//trust relations
		sim.traderInfoNetworkType = NetworkType.NETWORK_RANDOM;
		setupConfiguration(sim);

		ui = new MaghribiSimUi(sim);
		stepUi(ui, experimentalRounds, Stats.getGlobalSubfolderForOutput());
		
		//FIXED
		sim = new MaghribiSim(seed, showUi);
		MicroBootProperties.setRandomNumberGeneratorSeed(sim.seed());
		
		//trust relations
		sim.traderInfoNetworkType = NetworkType.NETWORK_FIXED;
		setupConfiguration(sim);

		ui = new MaghribiSimUi(sim);
		stepUi(ui, experimentalRounds, Stats.getGlobalSubfolderForOutput());
		
		//FIXED_MUTUAL
		sim = new MaghribiSim(seed, showUi);
		MicroBootProperties.setRandomNumberGeneratorSeed(sim.seed());
		
		//trust relations
		sim.traderInfoNetworkType = NetworkType.NETWORK_FIXED_MUTUAL;
		setupConfiguration(sim);

		ui = new MaghribiSimUi(sim);
		stepUi(ui, experimentalRounds, Stats.getGlobalSubfolderForOutput());
		
		//WATTS_STROGATZ
		sim = new MaghribiSim(seed, showUi);
		MicroBootProperties.setRandomNumberGeneratorSeed(sim.seed());
		
		//trust relations
		sim.traderInfoNetworkType = NetworkType.NETWORK_WATTS_STROGATZ;
		setupConfiguration(sim);

		ui = new MaghribiSimUi(sim);
		stepUi(ui, experimentalRounds, Stats.getGlobalSubfolderForOutput());
		
		//BARABASI-ALBERT
		sim = new MaghribiSim(seed, showUi);
		MicroBootProperties.setRandomNumberGeneratorSeed(sim.seed());
		
		//trust relations
		sim.traderInfoNetworkType = NetworkType.NETWORK_BARABASI_ALBERT;
		setupConfiguration(sim);

		ui = new MaghribiSimUi(sim);
		stepUi(ui, experimentalRounds, Stats.getGlobalSubfolderForOutput());
	}

	

}
