package fuzzyMaghribis.network;

import java.util.Arrays;
import org.graphstream.algorithm.Algorithm;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import static org.graphstream.algorithm.Toolkit.*;

public class DegreesAlgorithm implements Algorithm {
	Graph theGraph;
	int minDegree, maxDegree;
	float avgDegree;

	public void init(Graph graph) {
		theGraph = graph;
	}

	public void compute() {
		avgDegree = 0;
		minDegree = Integer.MAX_VALUE;
		maxDegree = 0;

		for (Node n : theGraph.getEachNode()) {
			int deg = n.getDegree();

			minDegree = Math.min(minDegree, deg);
			maxDegree = Math.max(maxDegree, deg);
			avgDegree += deg;
		}
		avgDegree /= ((float) theGraph.getNodeCount());
	}

	@Override
	public String toString() {
		return "DegreesAlgorithm [theGraph=" + theGraph + ", minDegree="
				+ minDegree + ", maxDegree=" + maxDegree + ", avgDegree="
				+ avgDegree + ", degreeDistribution="
				+ Arrays.toString(degreeDistribution(theGraph)) + ", averageDeviation="
				+ degreeAverageDeviation(theGraph) + "]";
	}

	public GraphInformation getResults() {
		return new GraphInformation(minDegree, maxDegree, avgDegree, theGraph.getNodeCount(), theGraph.getEdgeCount(), averageClusteringCoefficient(theGraph), diameter(theGraph), degreeDistribution(theGraph), degreeAverageDeviation(theGraph));
	}

	public int getMaxDegree() {
		return maxDegree;
	}

	public int getMinDegree() {
		return minDegree;
	}

	public float getAverageDegree() {
		return avgDegree;
	}
	
	public int[] getDegreeDistribution(){
		System.out.println("Calculated degree distribution.");
		return degreeDistribution(theGraph);
	}
	
	public double getAverageDegreeDistribution(){
		return degreeAverageDeviation(theGraph);
	}
	
}