package fuzzyMaghribis.network;

import java.util.ArrayList;
import org.nzdis.micro.MTConnector;
import org.nzdis.micro.MicroMessage;
import org.sofosim.environment.GridSim;
import fuzzyMaghribis.MaghribiSim;
import fuzzyMaghribis.Trader;

public class FixedNetwork implements Network{

	private Trader owner = null;
	private GridSim sim = null;
	private ArrayList<String> traders = new ArrayList<String>();
	
	@Override
	public void initialize(Trader requester, GridSim sim) {
		if(!isInitialized()){
			this.owner = requester;
			this.sim = sim;
			//ensure that we only request advice from other traders excluding self
			for(String traderName: sim.agentDirectory.keySet()){
				if(traderName.startsWith(MaghribiSim.TRADER_PREFIX) && !traderName.equals(this.owner.me())){
					traders.add(traderName);
				}
			}
			System.out.println("Initialized FIXED network for " + requester.me());
		}
	}

	@Override
	public boolean isInitialized() {
		return (this.owner != null && this.sim != null);
	}

	@Override
	public Advice getAdvice(String candidate) {
		Advice advice = new Advice(owner.me(), candidate);
		if(owner.traderConnections.size() == MaghribiSim.maxNumberOfTraderTrustRelations){
			String advisorCandidate = owner.traderConnections.get(sim.random().nextInt(owner.traderConnections.size()));
			owner.print("Picked for requesting advice based on established network: " + advisorCandidate);
			return ((Trader)sim.agentDirectory.get(advisorCandidate)).getAdvice(advice);
		}
		//pick random advisor
		String advisor = traders.get(sim.random().nextInt(traders.size()));
		owner.print("Picked advisor randomly: " + advisor);
		return ((Advisor)sim.agentDirectory.get(advisor)).getAdvice(advice);
	}

	@Override
	public Integer getNumberOfNodes() {
		return traders.size() + 1;
	}

	@Override
	public Integer getNumberOfLinks() {
		int count = 0;
		for(int i=0; i<traders.size(); i++){
			count += ((Trader)sim.agentDirectory.get(traders.get(i))).traderConnections.size();
		}
		try{
			count += owner.traderConnections.size();
		} catch(NullPointerException e){
			
		}
		//return half as we are counting links twice (undirected!)
		return count / 2;
	}

	@Override
	public GraphInformation getDegreeInformation() {
		int minDegree = Integer.MAX_VALUE;
		int maxDegree = 0;
		float count = 0f;
		ArrayList<Double> degreeList = new ArrayList<Double>();
		for(int i=0; i<traders.size(); i++){
			int degree = ((Trader)sim.agentDirectory.get(traders.get(i))).traderConnections.size();
			degreeList.add(Double.parseDouble(String.valueOf(degree)));
			minDegree = Math.min(minDegree, degree);
			maxDegree = Math.max(maxDegree, degree);
			count += degree;
		}
		float avgDegree = count / (float)(traders.size() + 1);
		return new GraphInformation(minDegree, maxDegree, avgDegree, getNumberOfNodes(), getNumberOfLinks(), degreeList);
	}
	
	@Override
	public String getGraphType() {
		return MaghribiSim.traderInfoNetworkType;
	}

	@Override
	public void reset() {
		this.owner = null;
		this.sim = null;
		this.traders.clear();
	}

	@Override
	public void shareCheaterInformation(MicroMessage message, int numberOfOthers) {
		message.setSender(owner.me());
		MTConnector.sendFuzzycast(message, 1.0, false, true, owner.traderConnections);
	}

}
