package fuzzyMaghribis.network;

import org.nzdis.micro.MTConnector;
import org.nzdis.micro.MicroMessage;
import org.sofosim.environment.GridSim;
import fuzzyMaghribis.MaghribiSim;
import fuzzyMaghribis.Trader;

public class GeneratedNetwork extends GraphGenerators implements Network{

	private Trader trader = null;
	private GridSim sim = null;
	
	@Override
	public void initialize(Trader requester, GridSim sim) {
		if(requester == null || sim == null){
			throw new RuntimeException("Trader or sim is null - trader: " + trader + ", sim: " + sim);
		}
		this.trader = requester;
		this.sim = sim;
		defineNode(new TraderConnectionModifier(requester, sim));
	}

	@Override
	public boolean isInitialized() {
		return trader != null;
	}

	@Override
	public synchronized Advice getAdvice(String candidate) {
		Trader target = (Trader)sim.agentDirectory.get(trader.traderConnections.get(sim.random().nextInt(trader.traderConnections.size())));
		trader.print("Selected advisor " + target.me() + " out of collection of " + trader.traderConnections.size());// + ": " + trader.traderConnections);
		Advice advice = new Advice(candidate);
		advice.setSender(this.trader.me());
		if(!isInitialized()){
			throw new RuntimeException("Small world/scale free Network for " + this.trader.me() + " has not been initialized.");
		}
		return target.getAdvice(advice);
	}

	@Override
	public Integer getNumberOfNodes() {
		return graph.getNodeCount();
	}

	@Override
	public Integer getNumberOfLinks() {
		return graph.getEdgeCount();
	}

	@Override
	public GraphInformation getDegreeInformation() {
		DegreesAlgorithm alg = new DegreesAlgorithm();
		alg.init(graph);
		alg.compute();
		return alg.getResults();
	}

	@Override
	public String getGraphType() {
		return MaghribiSim.traderInfoNetworkType;
	}

	@Override
	public void reset() {
		this.trader = null;
		this.sim = null;
		resetGraphGenerator();
	}

	@Override
	public void shareCheaterInformation(MicroMessage message, int numberOfOthers) {
		message.setSender(trader.me());
		MTConnector.sendFuzzycast(message, 1.0, false, true, trader.traderConnections);
	}

}
