package fuzzyMaghribis.network;

import java.util.ArrayList;

public class GraphInformation {

	public final int minDegree;
	public final int maxDegree;
	public final float avgDegree;
	public final int numberOfNodes;
	public final int numberOfEdges;
	public final Double avgClusterCoefficient;
	public final Double diameter;
	public final int[] degreeDistribution;
	public final ArrayList<Double> degreeDistributionArray;
	public final Double averageDegreeDistribution;
	
	public GraphInformation(int minDegree, int maxDegree, float avgDegree, int numberOfNodes, int numberOfEdges) {
		super();
		this.minDegree = minDegree;
		this.maxDegree = maxDegree;
		this.avgDegree = avgDegree;
		this.numberOfNodes = numberOfNodes;
		this.numberOfEdges = numberOfEdges;
		this.avgClusterCoefficient = null;
		this.diameter = null;
		this.degreeDistribution = null;
		this.averageDegreeDistribution = null;
		this.degreeDistributionArray = null;
	}
	
	public GraphInformation(int minDegree, int maxDegree, float avgDegree, int numberOfNodes, int numberOfEdges, ArrayList<Double> degreeDistribution) {
		super();
		this.minDegree = minDegree;
		this.maxDegree = maxDegree;
		this.avgDegree = avgDegree;
		this.numberOfNodes = numberOfNodes;
		this.numberOfEdges = numberOfEdges;
		this.avgClusterCoefficient = null;
		this.diameter = null;
		this.degreeDistribution = null;
		this.averageDegreeDistribution = null;
		this.degreeDistributionArray = degreeDistribution;
	}
	
	public GraphInformation(int minDegree, int maxDegree, float avgDegree, int numberOfNodes, int numberOfEdges, double avgClusterCoefficient) {
		super();
		this.minDegree = minDegree;
		this.maxDegree = maxDegree;
		this.avgDegree = avgDegree;
		this.numberOfNodes = numberOfNodes;
		this.numberOfEdges = numberOfEdges;
		this.avgClusterCoefficient = avgClusterCoefficient;
		this.diameter = null;
		this.degreeDistribution = null;
		this.averageDegreeDistribution = null;
		this.degreeDistributionArray = null;
	}
	
	public GraphInformation(int minDegree, int maxDegree, float avgDegree, int numberOfNodes, int numberOfEdges, double avgClusterCoefficient, double diameter, double averageDegreeDistribution) {
		super();
		this.minDegree = minDegree;
		this.maxDegree = maxDegree;
		this.avgDegree = avgDegree;
		this.numberOfNodes = numberOfNodes;
		this.numberOfEdges = numberOfEdges;
		this.avgClusterCoefficient = avgClusterCoefficient;
		this.diameter = diameter;
		this.degreeDistribution = null;
		this.averageDegreeDistribution = averageDegreeDistribution;
		this.degreeDistributionArray = null;
	}
	
	public GraphInformation(int minDegree, int maxDegree, float avgDegree, int numberOfNodes, int numberOfEdges, double avgClusterCoefficient, double diameter, int[] degreeDistribution, double averageDegreeDistribution) {
		super();
		this.minDegree = minDegree;
		this.maxDegree = maxDegree;
		this.avgDegree = avgDegree;
		this.numberOfNodes = numberOfNodes;
		this.numberOfEdges = numberOfEdges;
		this.avgClusterCoefficient = avgClusterCoefficient;
		this.diameter = diameter;
		this.degreeDistribution = degreeDistribution;
		this.averageDegreeDistribution = averageDegreeDistribution;
		this.degreeDistributionArray = null;
	}
	
	public GraphInformation(int minDegree, int maxDegree, float avgDegree, int numberOfNodes, int numberOfEdges, double avgClusterCoefficient, double diameter, ArrayList<Double> degreeDistribution, double averageDegreeDistribution) {
		super();
		this.minDegree = minDegree;
		this.maxDegree = maxDegree;
		this.avgDegree = avgDegree;
		this.numberOfNodes = numberOfNodes;
		this.numberOfEdges = numberOfEdges;
		this.avgClusterCoefficient = avgClusterCoefficient;
		this.diameter = diameter;
		this.degreeDistribution = null;
		this.degreeDistributionArray = degreeDistribution;
		this.averageDegreeDistribution = averageDegreeDistribution;
	}

	@Override
	public String toString() {
		StringBuffer buf = null;
		if(degreeDistribution != null){
			buf = new StringBuffer("Distribution raw data: ");
			for(int i = 0; i < degreeDistribution.length; i++){
				buf.append("Count for degree ").append(i).append(": ");
				buf.append(degreeDistribution[i]);
				if(i < degreeDistribution.length - 1){
					buf.append(", ");
				}
			}
		}
		return "DegreeInformation:\nminDegree=" + minDegree + ",\nmaxDegree="
				+ maxDegree + ",\navgDegree=" + avgDegree + ",\nnumberOfNodes="
				+ numberOfNodes + ",\nnumberOfEdges=" + numberOfEdges 
				+ "\navgClusterCoefficient=" + avgClusterCoefficient
				+ "\ndiameter=" + diameter 
				+ "\ndegreeDistribution=" 
				+ (degreeDistribution == null ? degreeDistribution : buf.toString())
				+ "\naverageDegreeDistribution=" + averageDegreeDistribution;
	}

}
