package fuzzyMaghribis.network;

import org.nzdis.micro.MicroMessage;
import org.sofosim.environment.GridSim;
import fuzzyMaghribis.Trader;

public interface Network {

	/**
	 * Initializes a connection network for a given 
	 * trader and simulation
	 * @param requester
	 * @param sim
	 */
	void initialize(Trader requester, GridSim sim);
	
	/**
	 * Indicates if network has been initialized
	 * @return
	 */
	boolean isInitialized();
	
	/**
	 * shares information about cheating traders as well 
	 * as cheaters over the network
	 */
	void shareCheaterInformation(MicroMessage message, int numberOfOthers);
	
	/**
	 * Returns advice for a given candidate
	 * @param candidate
	 * @return
	 */
	Advice getAdvice(String candidate);
	
	/**
	 * Returns number of network nodes.
	 * @return
	 */
	Integer getNumberOfNodes();
	
	/**
	 * Returns number of network edges.
	 * @return
	 */
	Integer getNumberOfLinks();
	
	/**
	 * Returns some statistics about network.
	 * @return
	 */
	GraphInformation getDegreeInformation();
	
	/**
	 * Returns the types of the graph as specified
	 * by the experimenter.
	 * @return
	 */
	String getGraphType();
	
	/**
	 * Resets internal data structures for reinstantiation. 
	 * Important when using generated graphs.
	 */
	void reset();
	
}
