package fuzzyMaghribis.network;

import java.util.ArrayList;

import org.nzdis.micro.MTConnector;
import org.nzdis.micro.MicroMessage;
import org.sofosim.environment.GridSim;
import fuzzyMaghribis.MaghribiSim;
import fuzzyMaghribis.Trader;

public class RandomNetwork implements Network {

	private String owner = null;
	private GridSim sim = null;
	private ArrayList<String> traders = new ArrayList<String>();
	
	@Override
	public void initialize(Trader requester, GridSim sim) {
		if(!isInitialized()){
			if(requester == null || sim == null){
				throw new RuntimeException("Random Network initialization requires specification of owner and simulation instance.");
			}
			this.sim = sim;
			this.owner = requester.me();
			//ensure that we only request advice from other traders excluding self
			for(String traderName: sim.agentDirectory.keySet()){
				if(traderName.startsWith(MaghribiSim.TRADER_PREFIX) && !traderName.equals(this.owner)){
					traders.add(traderName);
				}
			}
			System.out.println("Initialized RANDOM network for " + requester.me());
		}
	}
	
	@Override
	public boolean isInitialized(){
		return (sim != null && !traders.isEmpty());
	}

	@Override
	public Advice getAdvice(String adviceObject) {
		Advice advice = new Advice(adviceObject);
		advice.setSender(owner);
		if(!isInitialized()){
			throw new RuntimeException("RandomNetwork for " + this.owner + " has not been initialized.");
		}
		String advisor = traders.get(sim.random().nextInt(traders.size()));
		return ((Advisor)sim.agentDirectory.get(advisor)).getAdvice(advice);
	}

	@Override
	public Integer getNumberOfNodes() {
		return traders.size() + 1;
	}

	@Override
	public Integer getNumberOfLinks() {
		int count = 0;
		for(int i=0; i<traders.size(); i++){
			count += ((Trader)sim.agentDirectory.get(traders.get(i))).traderConnections.size();
		}
		try{
			count += ((Trader)sim.agentDirectory.get(owner)).traderConnections.size();
		} catch(NullPointerException e){
			
		}
		return count;
	}

	@Override
	public GraphInformation getDegreeInformation() {
		int minDegree = Integer.MAX_VALUE;
		int maxDegree = 0;
		float count = 0f;
		//ArrayList<Double> degreeList = new ArrayList<Double>();
		for(int i=0; i<traders.size(); i++){
			int degree = ((Trader)sim.agentDirectory.get(traders.get(i))).traderConnections.size();
			//degreeList.add(Double.parseDouble(String.valueOf(degree)));
			minDegree = Math.min(minDegree, degree);
			maxDegree = Math.max(maxDegree, degree);
			count += degree;
		}
		float avgDegree = count / (float)(traders.size() + 1);
		return new GraphInformation(minDegree, maxDegree, avgDegree, getNumberOfNodes(), getNumberOfLinks());
	}

	@Override
	public String getGraphType() {
		return MaghribiSim.traderInfoNetworkType;
	}

	@Override
	public void reset() {
		this.owner = null;
		this.sim = null;
		this.traders.clear();
	}

	@Override
	public void shareCheaterInformation(MicroMessage message, int numberOfOthers) {
		message.setSender(owner);
		MTConnector.sendRandomcast(message, numberOfOthers, false, true, traders);
	}

}
